@@ -246,7 +246,7 @@ disk_get_fullpath(DEVICE * dev, const char * path)
 	}
 	if (len > 0 && fullpath[len - 1] == '/')
 		fullpath[len - 1] = '\0';
-		
+
 	return fullpath;
 }
 
@@ -303,7 +303,7 @@ disk_create_fullpath(IRP * irp, FILE_INFO * finfo, const char * fullpath)
 		}
 
 		if ((irp->desiredAccess & GENERIC_ALL)
-		    || (irp->desiredAccess & GENERIC_WRITE)
+			|| (irp->desiredAccess & GENERIC_WRITE)
 			|| (irp->desiredAccess & FILE_WRITE_DATA)
 			|| (irp->desiredAccess & FILE_APPEND_DATA))
 		{
@@ -813,7 +813,7 @@ disk_query_directory(IRP * irp, uint8 initialQuery, const char * path)
 	if (pdirent == NULL)
 	{
 		return RD_STATUS_NO_MORE_FILES;
-	}		
+	}
 
 	memset(&file_stat, 0, sizeof(struct stat));
 	p = malloc(strlen(finfo->fullpath) + strlen(pdirent->d_name) + 2);
@@ -918,6 +918,13 @@ disk_free(DEVICE * dev)
 	return 0;
 }
 
+static int
+disk_get_fd(IRP * irp)
+{
+	FILE_INFO * finfo = disk_get_file_info(irp->dev, irp->fileID);
+	return finfo->file;
+}
+
 static SERVICE *
 disk_register_service(PDEVMAN pDevman, PDEVMAN_ENTRY_POINTS pEntryPoints)
 {
@@ -938,6 +945,9 @@ disk_register_service(PDEVMAN pDevman, PDEVMAN_ENTRY_POINTS pEntryPoints)
 	srv->lock_control = disk_lock_control;
 	srv->free = disk_free;
 	srv->type = RDPDR_DTYP_FILESYSTEM;
+	srv->get_event = NULL;
+	srv->file_descriptor = disk_get_fd;
+	srv->get_timeouts = NULL;
 
 	return srv;
 }
