@@ -207,7 +207,7 @@ printer_process_data(SERVICE * srv, int type, const char * data, int data_len)
 	return 0;
 }
 
-static SERVICE * 
+static SERVICE *
 printer_register_service(PDEVMAN pDevman, PDEVMAN_ENTRY_POINTS pEntryPoints)
 {
 	SERVICE * srv;
@@ -228,6 +228,9 @@ printer_register_service(PDEVMAN pDevman, PDEVMAN_ENTRY_POINTS pEntryPoints)
 	srv->free = printer_free;
 	srv->process_data = printer_process_data;
 	srv->type = RDPDR_DTYP_PRINT;
+	srv->get_event = NULL;
+	srv->file_descriptor = NULL;
+	srv->get_timeouts = NULL;
 
 	return srv;
 }
