@@ -27,6 +27,7 @@
 #include <pthread.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <sys/select.h>
 
 #include "rdpdr_types.h"
 #include "rdpdr_main.h"
@@ -34,6 +35,9 @@
 #include "rdpdr_capabilities.h"
 #include "devman.h"
 #include "irp.h"
+#include "irp_queue.h"
+
+#define MAX(x,y)             (((x) > (y)) ? (x) : (y))
 
 /* called by main thread
    add item to linked list and inform worker thread that there is data */
@@ -247,6 +251,181 @@ rdpdr_send_device_list_announce_request(rdpdrPlugin * plugin)
 	return 0;
 }
 
+static void
+rdpdr_add_async_irp(rdpdrPlugin * plugin, IRP * irp, char * data, int data_size)
+{
+	fd_set *fds = NULL;
+	uint32 timeout = 0, itv_timeout = 0;
+
+	irp->length = GET_UINT32(data, 0); /* length */
+	irp->offset = GET_UINT64(data, 4); /* offset */
+	irp->inputBuffer = NULL;
+
+	if (irp->majorFunction == IRP_MJ_WRITE)
+	{
+		fds = &plugin->writefds;
+		irp->inputBuffer = malloc(data_size - 32);
+		memcpy(irp->inputBuffer, data + 32, data_size - 32);
+		irp->inputBufferLength = irp->length;
+	}
+	else
+		fds = &plugin->readfds;
+
+	if (irp->dev->service->type == RDPDR_DTYP_SERIAL)
+		irp_get_timeouts(irp, &timeout, &itv_timeout);
+
+	/* Check if io request timeout is smaller than current (but not 0). */
+	if (timeout && (plugin->select_timeout == 0 || timeout < plugin->select_timeout))
+	{
+		plugin->select_timeout = timeout;
+		plugin->tv.tv_sec = plugin->select_timeout / 1000;
+		plugin->tv.tv_usec = (plugin->select_timeout % 1000) * 1000;
+	}
+	if (itv_timeout && (plugin->select_timeout == 0 || itv_timeout < plugin->select_timeout))
+	{
+		plugin->select_timeout = itv_timeout;
+		plugin->tv.tv_sec = plugin->select_timeout / 1000;
+		plugin->tv.tv_usec = (plugin->select_timeout % 1000) * 1000;
+	}
+
+	LLOGLN(10, ("RDPDR adding async irp fd %d", irp_file_descriptor(irp)));
+	irp->ioStatus = RD_STATUS_PENDING;
+	irp_queue_push(plugin->queue, irp);
+
+	if (irp_file_descriptor(irp) >= 0)
+	{
+		FD_SET(irp_file_descriptor(irp), fds);
+		plugin->nfds = MAX(plugin->nfds, irp_file_descriptor(irp));
+	}
+}
+
+static void
+rdpdr_abort_single_io(rdpdrPlugin * plugin, uint32 fd, uint8 abortCode)
+{
+	IRP * pending = NULL;
+	char * out;
+	int out_size, error;
+	int major = 0;
+
+	if (abortCode == RDPDR_ABORT_IO_READ)
+		major = IRP_MJ_READ;
+	else if (abortCode == RDPDR_ABORT_IO_WRITE)
+		major = IRP_MJ_WRITE;
+
+	pending = irp_queue_first(plugin->queue);
+	while (pending)
+	{
+		if (irp_file_descriptor(pending) == fd && pending->majorFunction == major)
+		{
+			pending->ioStatus = RD_STATUS_CANCELLED;
+			out = irp_output_device_io_completion(pending, &out_size);
+			error = plugin->ep.pVirtualChannelWrite(plugin->open_handle, out, out_size, out);
+			if (error != CHANNEL_RC_OK)
+				LLOGLN(0, ("rdpdr_check_fds: VirtualChannelWrite failed %d", error));
+			if (pending->outputBuffer)
+				free(pending->outputBuffer);
+			irp_queue_remove(plugin->queue, pending);
+
+			return;
+		}
+
+		pending = irp_queue_next(plugin->queue, pending);
+	}
+}
+
+static void
+rdpdr_abort_ios(rdpdrPlugin * plugin)
+{
+	IRP * pending = NULL;
+	char * out;
+	int out_size, error;
+
+	while (!irp_queue_empty(plugin->queue))
+	{
+		pending = irp_queue_first(plugin->queue);
+		pending->ioStatus = RD_STATUS_SUCCESS;
+		out = irp_output_device_io_completion(pending, &out_size);
+		error = plugin->ep.pVirtualChannelWrite(plugin->open_handle, out, out_size, out);
+		if (error != CHANNEL_RC_OK)
+				LLOGLN(0, ("rdpdr_check_fds: VirtualChannelWrite failed %d", error));
+		if (pending->outputBuffer)
+			free(pending->outputBuffer);
+		irp_queue_pop(plugin->queue);
+	}
+}
+
+static int
+rdpdr_check_fds(rdpdrPlugin * plugin)
+{
+	IRP * pending = NULL, * prev = NULL;
+	char * out;
+	int out_size, error;
+	uint32 result;
+
+	if (select(plugin->nfds + 1, &plugin->readfds, &plugin->writefds, NULL, &plugin->tv) <= 0)
+		return 0;
+
+	memset(&plugin->tv, 0, sizeof(struct timeval));
+
+	/* scan every pending */
+	pending = irp_queue_first(plugin->queue);
+	while (pending)
+	{
+		int isset = 0;
+		prev = pending;
+		switch (pending->majorFunction)
+		{
+			case IRP_MJ_READ:
+				if (FD_ISSET(irp_file_descriptor(pending), &plugin->readfds))
+				{
+					irp_process_read_request(pending, NULL, 0);
+					isset = 1;
+				}
+				break;
+
+			case IRP_MJ_WRITE:
+				if (FD_ISSET(irp_file_descriptor(pending), &plugin->writefds))
+				{
+					irp_process_write_request(pending, NULL, 0);
+					isset = 1;
+				}
+				break;
+
+			case IRP_MJ_DEVICE_CONTROL:
+				if (irp_get_event(pending, &result))
+				{
+					pending->ioStatus = RD_STATUS_SUCCESS;
+					out = irp_output_device_io_completion(pending, &out_size);
+					error = plugin->ep.pVirtualChannelWrite(plugin->open_handle, out, out_size, out);
+					if (pending->outputBuffer)
+						free(pending->outputBuffer);
+					isset = 2;
+				}
+				break;
+
+			default:
+				LLOGLN(1, ("rdpdr_check_fds: no request found"));
+				break;
+		}
+
+		if (isset == 1)
+		{
+			out = irp_output_device_io_completion(pending, &out_size);
+			error = plugin->ep.pVirtualChannelWrite(plugin->open_handle, out, out_size, out);
+			if (error != CHANNEL_RC_OK)
+				LLOGLN(0, ("rdpdr_check_fds: VirtualChannelWrite failed %d", error));
+
+			if (pending->inputBuffer)
+				free(pending->inputBuffer);
+		}
+		pending = irp_queue_next(plugin->queue, pending);
+		if (isset)
+			irp_queue_remove(plugin->queue, prev);
+	}
+
+	return 1;
+}
+
 static void
 rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 {
@@ -255,10 +434,12 @@ rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 	char * out;
 	int out_size;
 	int error;
+	uint32 result;
 
 	memset((void*)&irp, '\0', sizeof(IRP));
 
 	irp.ioStatus = RD_STATUS_SUCCESS;
+	irp.abortIO = RDPDR_ABORT_IO_NONE;
 
 	/* Device I/O Request Header */
 	deviceID = GET_UINT32(data, 0); /* deviceID */
@@ -268,6 +449,34 @@ rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 	irp.minorFunction = GET_UINT32(data, 16); /* minorFunction */
 
 	irp.dev = devman_get_device_by_id(plugin->devman, deviceID);
+	switch (irp.dev->service->type)
+	{
+		case RDPDR_DTYP_SERIAL:
+			irp.rwBlocking = 0;
+			break;
+
+		/* parallel, filesystem and smart card are supposed to be non-blocking
+			but my goal so far is to deal only with the serial stuff */
+		case RDPDR_DTYP_PARALLEL:
+			irp.rwBlocking = 1;
+			break;
+
+		case RDPDR_DTYP_FILESYSTEM:
+			irp.rwBlocking = 1;
+			break;
+
+		case RDPDR_DTYP_SMARTCARD:
+			irp.rwBlocking = 1;
+			break;
+
+		case RDPDR_DTYP_PRINT:
+			irp.rwBlocking = 1;
+			break;
+
+		default:
+			irp.rwBlocking = 1;
+			break;
+	}
 
 	LLOGLN(10, ("IRP MAJOR: %d MINOR: %d", irp.majorFunction, irp.minorFunction));
 
@@ -285,12 +494,18 @@ rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 
 		case IRP_MJ_READ:
 			LLOGLN(10, ("IRP_MJ_READ"));
-			irp_process_read_request(&irp, &data[20], data_size - 20);
+			if (irp.rwBlocking)
+				irp_process_read_request(&irp, &data[20], data_size - 20);
+			else
+				rdpdr_add_async_irp(plugin, &irp, &data[20], data_size - 20);
 			break;
 
 		case IRP_MJ_WRITE:
 			LLOGLN(10, ("IRP_MJ_WRITE"));
-			irp_process_write_request(&irp, &data[20], data_size - 20);
+			if (irp.rwBlocking)
+				irp_process_write_request(&irp, &data[20], data_size - 20);
+			else
+				rdpdr_add_async_irp(plugin, &irp, &data[20], data_size - 20);
 			break;
 
 		case IRP_MJ_QUERY_INFORMATION:
@@ -316,6 +531,8 @@ rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 		case IRP_MJ_DEVICE_CONTROL:
 			LLOGLN(10, ("IRP_MJ_DEVICE_CONTROL"));
 			irp_process_device_control_request(&irp, &data[20], data_size - 20);
+			if (irp.ioStatus == RD_STATUS_PENDING)
+				irp_queue_push(plugin->queue, &irp);
 			break;
 
 		case IRP_MJ_LOCK_CONTROL:
@@ -329,7 +546,20 @@ rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 			break;
 	}
 
-	if (irp.ioStatus != RD_STATUS_PENDING)
+	if (irp.abortIO)
+	{
+		if (irp.abortIO & RDPDR_ABORT_IO_WRITE)
+			rdpdr_abort_single_io(plugin, irp_file_descriptor(&irp), RDPDR_ABORT_IO_WRITE);
+		if (irp.abortIO & RDPDR_ABORT_IO_READ)
+			rdpdr_abort_single_io(plugin, irp_file_descriptor(&irp), RDPDR_ABORT_IO_READ);
+	}
+
+	if (irp.ioStatus == RD_STATUS_PENDING && irp.rwBlocking)
+	{
+		LLOGLN(10, ("IRP enqueue event"));
+		irp_queue_push(plugin->queue, &irp);
+	}
+	else if (irp.ioStatus != RD_STATUS_PENDING)
 	{
 		out = irp_output_device_io_completion(&irp, &out_size);
 		error = plugin->ep.pVirtualChannelWrite(plugin->open_handle, out, out_size, out);
@@ -338,9 +568,29 @@ rdpdr_process_irp(rdpdrPlugin * plugin, char* data, int data_size)
 			LLOGLN(0, ("rdpdr_process_irp: "
 				"VirtualChannelWrite failed %d", error));
 		}
+		if (irp.outputBuffer)
+		{
+			free(irp.outputBuffer);
+			irp.outputBuffer = NULL;
+			irp.outputBufferLength = 0;
+		}
+	}
+
+	if (irp_get_event(&irp, &result) && irp.rwBlocking)
+	{
+		LLOGLN(10, ("IRP process pending events"));
+		IRP * pending = 0;
+		while (!irp_queue_empty(plugin->queue))
+		{
+			pending = irp_queue_first(plugin->queue);
+			pending->ioStatus = RD_STATUS_SUCCESS;
+			out = irp_output_device_io_completion(pending, &out_size);
+			error = plugin->ep.pVirtualChannelWrite(plugin->open_handle, out, out_size, out);
+			if (pending->outputBuffer)
+				free(pending->outputBuffer);
+			irp_queue_pop(plugin->queue);
+		}
 	}
-	if (irp.outputBuffer)
-		free(irp.outputBuffer);
 }
 
 static int
@@ -535,7 +785,7 @@ thread_func(void * arg)
 	int numobj;
 
 	plugin = (rdpdrPlugin *) arg;
-
+	plugin->queue = irp_queue_new();
 	plugin->thread_status = 1;
 	LLOGLN(10, ("thread_func: in"));
 
@@ -546,6 +796,15 @@ thread_func(void * arg)
 		numobj = 2;
 		wait_obj_select(listobj, numobj, NULL, 0, -1);
 
+		plugin->nfds = 1;
+		FD_ZERO(&plugin->readfds);
+		FD_ZERO(&plugin->writefds);
+
+		/* default timeout */
+		plugin->tv.tv_sec = 0;
+		plugin->tv.tv_usec = 20;
+		plugin->select_timeout = 0;
+
 		if (wait_obj_is_set(plugin->term_event))
 		{
 			break;
@@ -556,10 +815,19 @@ thread_func(void * arg)
 			/* process data in */
 			thread_process_data(plugin);
 		}
+
+		rdpdr_check_fds(plugin);
+
+		if (irp_queue_size(plugin->queue))
+		{
+			rdpdr_abort_ios(plugin);
+/*			wait_obj_set(plugin->data_in_event);*/
+		}
 	}
 
 	LLOGLN(10, ("thread_func: out"));
 	plugin->thread_status = -1;
+	irp_queue_free(plugin->queue);
 	return 0;
 }
 
@@ -716,7 +984,8 @@ VirtualChannelEntry(PCHANNEL_ENTRY_POINTS pEntryPoints)
 	plugin->ep = *pEntryPoints;
 
 	memset(&(plugin->channel_def), 0, sizeof(plugin->channel_def));
-	plugin->channel_def.options = CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP;
+	plugin->channel_def.options = CHANNEL_OPTION_INITIALIZED |
+		CHANNEL_OPTION_ENCRYPT_RDP | CHANNEL_OPTION_COMPRESS_RDP;
 	strcpy(plugin->channel_def.name, "rdpdr");
 
 	plugin->mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
@@ -740,6 +1009,8 @@ VirtualChannelEntry(PCHANNEL_ENTRY_POINTS pEntryPoints)
 	plugin->devman = devman_new(data);
 	devman_load_device_service(plugin->devman, "disk");
 	devman_load_device_service(plugin->devman, "printer");
+	devman_load_device_service(plugin->devman, "serial");
+	devman_load_device_service(plugin->devman, "parallel");
 
 	plugin->ep.pVirtualChannelInit(&plugin->chan_plugin.init_handle, &plugin->channel_def, 1,
 		VIRTUAL_CHANNEL_VERSION_WIN2000, InitEvent);
